//
//
// graph program
// 
// openGL version of the gc program 
//
// author
//     keiji imoto
//     department of information physiology
//     national institute for physiological sciences
//     okazaki, japan
//
// version
//     05-may-2011
//     18-feb-2015 updated for double-buffer rubberband
//                 different behaviors in MacOSX and Linux
//                 tested using MaxOSX 10.10 (Yosemite) and
//                        CentOS 7 
//

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cmath>
#include <vector>
#include <sys/stat.h>

#include "gr.h"

using namespace std;

#include "Graph.h"
#include "GraphView.h"

// global variables

CGraph g;
CGraphView gv;
Point p0, p1, p2;
bool rubberband;
int iDirect;


void windowDisplay(void) 
{    
    if (!rubberband) {
        int w = glutGet(GLUT_WINDOW_WIDTH); 
        int h = glutGet(GLUT_WINDOW_HEIGHT);
        
        glClear(GL_COLOR_BUFFER_BIT);
        gv.setPenColor(GC_GRAY);
        gv.drawAxis(w,h);
        gv.drawTrace();
	glutSwapBuffers();
    }
}


void windowReshape(int w, int h)
{
    glViewport(0, 0, w, h);
    glLoadIdentity();
    glOrtho(-0.5, w - 0.5, h - 0.5, -0.5, -1.0, 1.0);
}

//
//  mosue handler
//

void mouseButtonClick(int button, int state, int x, int y)  // mouse button down/up 
{
    if (state == GLUT_DOWN) {

	p1.x = p0.x = x;
	p1.y = p0.y = y;
    }else{      // state == GLUT_UP
	rubberband = false;
        if(iDirect==1){   // horizontal
            gv.mouseUp(p0.x, p0.y, p1.x, p1.y);
        }else{
            gv.mouseUpRight(p0.x, p0.y, p1.x, p1.y);
        }
        iDirect = 0;
    }    
    glutPostRedisplay();

}

void mouseMove(int x, int y)  // mouse move
{
    if(p1.x != x || p1.y != y){        
	glEnable(GL_COLOR_LOGIC_OP);
	glLogicOp(GL_XOR);
    
	if (rubberband) {
	    glBegin(GL_LINE_LOOP);
	    glVertex2i(p0.x, p0.y);
	    glVertex2i(p2.x, p0.y);
	    glVertex2i(p2.x, p2.y);
	    glVertex2i(p0.x, p2.y);
	    glEnd();
	    
	    if(iDirect==0){
		int ddx = fabs(p0.x-p1.x);
		int ddy = fabs(p0.y-p1.y);
		if(ddx > 10 || ddy > 10){
		    if(ddx > ddy){
			iDirect = 1;
		    }else if (ddx<ddy) {
			iDirect = 2;
		    } 
		}
	    }
	}
#ifdef Darwin
	p2.x = p1.x;
	p2.y = p1.y;
#endif
#ifdef Linux
	p2.x = x;
	p2.y = y;
#endif

	p1.x = x;
	p1.y = y;
           
	glBegin(GL_LINE_LOOP);
	glVertex2i(p0.x, p0.y);
	glVertex2i(p1.x, p0.y);
	glVertex2i(p1.x, p1.y);
	glVertex2i(p0.x, p1.y);
	glEnd();
	glutSwapBuffers();

	rubberband = true;   
	glDisable(GL_COLOR_LOGIC_OP);
	glLogicOp(GL_COPY);
    }
}

void keyboard(unsigned char key, int x, int y)
{

    switch (key) {
    case '\001':   // ctrl-A
	if(glutGetModifiers()==GLUT_ACTIVE_CTRL) {
	    gv.fullWaves();
	    glutPostRedisplay();
	}
	break;
    case 'a':
    case 'A':
	gv.fullWaves();
	glutPostRedisplay();
	break;
    case 'q':
    case 'Q':
    case '\033':  // esc
	exit(0);
    default:
	break;
    }
}

//void Idle(void) {
//	glutPostRedisplay();
//}

void init(void) {
    glClearColor(1.0, 1.0, 1.0, 0.);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-1., 1., -1., 1., -1., 1.);
}

//
//   menu handler
//
void menuPenColor(int val)
{
    if(val==1){
	gv.m_bMultiColor=true;
    }else{
	gv.m_bMultiColor=false;
	if(val==2){
	    gv.penColor = GC_RED;
	}else if(val==3){
	    gv.penColor = GC_GREEN;
	}else if(val==4){
	    gv.penColor = GC_BLUE;
	}         
    }
    glutPostRedisplay();
}

void menuAxisSelection(int val)
{
    if((val>=0) && (val<g.nwaves)){
	gv.changeMode(val);
	glutPostRedisplay();
    }
}

int main(int argc, char** argv) 
{    
    char s[20];	
    struct stat statInfo;

    if(argc<=1){
        printf("usage: gr fileName (numberOfTraces)\n");
        return 0;
    }else{
        if(stat(argv[1], &statInfo) != 0){
            printf("? file not found ?\n");
            return 0;
        }
        if(argc==2){
            g.OpenFile(argv[1], 0);
        }else{
            g.OpenFile(argv[1], atoi(argv[2]));
        }
    }
    if(g.nwaves == 0)return 0;
	
    g.findMinMax();
    gv.pGraph = &g;
	
    rubberband = false;
    iDirect = 0;
    int w = 800;
    int h = 400;
    
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
    glutInitWindowSize(w, h);
    glutCreateWindow(argv[1]);
    glClear(GL_COLOR_BUFFER_BIT);
    glutDisplayFunc(windowDisplay);    
    glutReshapeFunc(windowReshape);
    glutMouseFunc(mouseButtonClick);
    glutMotionFunc(mouseMove);
    glutKeyboardFunc(keyboard);
    
    // menu    
    int subMenu1 = glutCreateMenu(menuPenColor);
    glutAddMenuEntry("red", 2);
    glutAddMenuEntry("green", 3);
    glutAddMenuEntry("blue", 4);
	
    int subMenu2 = glutCreateMenu(menuAxisSelection);
    for(int i = 0;i<g.nwaves ; i++){
        sprintf(s,"tr %d", i);
        glutAddMenuEntry(s, i);
    }
		
    /*int mainMenu = */glutCreateMenu(menuPenColor);
    glutAddMenuEntry("multi-color", 1);
    glutAddSubMenu("mono-color", subMenu1);    
    glutAddSubMenu("x-axis", subMenu2);    
    glutAttachMenu(GLUT_RIGHT_BUTTON);
	
    // line spec	
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
    glEnable(GL_LINE_SMOOTH);
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
    glLineWidth (1.0);
	
    //	glutIdleFunc(Idle);
    
    init();
    glutMainLoop();
    
    return 0;
}



